@@ -1,4 +1,4 @@
-/* Copyright 2007, 2008 Matthew Barnes
+/* Copyright 2007-2010 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -19,6 +19,8 @@
 /**
  * SECTION: gva-common
  * @short_description: Common Definitions
+ *
+ * Common symbols used throughout GNOME Video Arcade.
  **/
 
 #ifndef GVA_COMMON_H
@@ -28,10 +30,13 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#include <glib.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
-#include <gtk/gtk.h>
+
+#if HAVE_GSTREAMER
+#include <gst/gst.h>
+#endif
 
 #include <sqlite3.h>
 #include <gconf/gconf-client.h>
@@ -39,12 +44,14 @@
 
 #define GVA_GCONF_PREFIX                "/apps/" PACKAGE
 #define GVA_GCONF_ALL_COLUMNS_KEY       GVA_GCONF_PREFIX "/all-columns"
+#define GVA_GCONF_AUTO_PLAY_KEY         GVA_GCONF_PREFIX "/auto-play"
 #define GVA_GCONF_AUTO_SAVE_KEY         GVA_GCONF_PREFIX "/auto-save"
 #define GVA_GCONF_COLUMNS_KEY           GVA_GCONF_PREFIX "/columns"
 #define GVA_GCONF_FAVORITES_KEY         GVA_GCONF_PREFIX "/favorites"
 #define GVA_GCONF_FULL_SCREEN_KEY       GVA_GCONF_PREFIX "/full-screen"
 #define GVA_GCONF_ERROR_FILE_KEY        GVA_GCONF_PREFIX "/error-file"
 #define GVA_GCONF_PROPERTIES_PAGE_KEY   GVA_GCONF_PREFIX "/properties-page"
+#define GVA_GCONF_PROPERTIES_PREFIX     GVA_GCONF_PREFIX "/properties"
 #define GVA_GCONF_SEARCH_KEY            GVA_GCONF_PREFIX "/search"
 #define GVA_GCONF_SELECTED_GAME_KEY     GVA_GCONF_PREFIX "/selected-game"
 #define GVA_GCONF_SELECTED_MATCH_KEY    GVA_GCONF_PREFIX "/selected-match"
