@@ -1,4 +1,4 @@
-/* Copyright 2007-2009 Matthew Barnes
+/* Copyright 2007-2010 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -107,6 +107,7 @@ main_entry_completion_match_selected_cb (GtkEntryCompletion *completion,
 
         gva_main_set_last_selected_match (column_name, search_text);
         gva_main_execute_search ();
+        gtk_widget_grab_focus (GVA_WIDGET_MAIN_TREE_VIEW);
 
         g_free (column_name);
         g_free (search_text);
@@ -184,18 +185,13 @@ gva_main_init (void)
 
         gconf_bridge_bind_window (
                 gconf_bridge_get (), GVA_GCONF_WINDOW_PREFIX,
-                GTK_WINDOW (GVA_WIDGET_MAIN_WINDOW), TRUE, FALSE);
+                GTK_WINDOW (GVA_WIDGET_MAIN_WINDOW), TRUE, TRUE);
 
         /* Initialize the search entry. */
         text = gva_main_get_last_search_text ();
         gtk_entry_set_text (GTK_ENTRY (GVA_WIDGET_MAIN_SEARCH_ENTRY), text);
         g_free (text);
 
-#if !GTK_CHECK_VERSION(2,14,0) && !defined HAVE_GNOME
-        /* Requires that we link against libgnome. */
-        gtk_action_set_sensitive (GVA_ACTION_CONTENTS, FALSE);
-#endif
-
         gtk_action_set_sensitive (GVA_ACTION_PROPERTIES, FALSE);
         gtk_action_set_sensitive (GVA_ACTION_RECORD, FALSE);
         gtk_action_set_sensitive (GVA_ACTION_SEARCH, FALSE);
@@ -355,9 +351,7 @@ gva_main_init_search_completion (GError **error)
         sqlite3_stmt *stmt;
         gint errcode;
 
-#if GTK_CHECK_VERSION(2,13,1)
         GList *list;
-#endif
 
         if (!gva_db_prepare (SQL_COMPLETION_LIST, &stmt, error))
                 return FALSE;
@@ -413,14 +407,9 @@ gva_main_init_search_completion (GError **error)
                 completion, "match-selected",
                 G_CALLBACK (main_entry_completion_match_selected_cb), NULL);
 
-        /* XXX GtkEntryCompletion does not implement get_cells().
-         *     http://bugzilla.gnome.org/show_bug.cgi?id=523787
-         *     Fixed in GTK+ 2.13.1. */
-#if GTK_CHECK_VERSION(2,13,1)
         list = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (completion));
         g_object_set (list->data, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
         g_list_free (list);
-#endif
 
         renderer = gtk_cell_renderer_text_new ();
         g_object_set (renderer, "sensitive", FALSE, "xalign", 1.0, NULL);
@@ -664,6 +653,25 @@ gva_main_statusbar_remove (guint context_id,
         gtk_statusbar_remove (statusbar, context_id, message_id);
 }
 
+/**
+ * gva_main_clear_search:
+ *
+ * Clears the search entry and, if the Search Results view is active,
+ * any results from the previous search.
+ **/
+void
+gva_main_clear_search (void)
+{
+        GtkWidget *widget;
+
+        widget = GVA_WIDGET_MAIN_SEARCH_ENTRY;
+        gtk_entry_set_text (GTK_ENTRY (widget), "");
+        gva_main_set_last_selected_match (NULL, NULL);
+
+        if (gva_tree_view_get_selected_view () == 2)
+                gva_main_execute_search ();
+}
+
 /**
  * gva_main_execute_search:
  *
@@ -723,8 +731,6 @@ gva_main_execute_search (void)
                         gtk_tree_path_free (path);
                 }
         }
-
-        gtk_widget_grab_focus (GTK_WIDGET (view));
 }
 
 /**
@@ -885,6 +891,48 @@ gva_main_search_entry_activate_cb (GtkEntry *entry)
 {
         gva_main_set_last_selected_match (NULL, NULL);
         gva_main_execute_search ();
+        gtk_widget_grab_focus (GVA_WIDGET_MAIN_TREE_VIEW);
+}
+
+/**
+ * gva_main_search_entry_changed_cb:
+ * @entry: the search entry
+ *
+ * Handler for #GtkEditable::changed signals to the search entry.
+ *
+ * Updates the sensitivity of the clear search icon.
+ **/
+void
+gva_main_search_entry_changed_cb (GtkEntry *entry)
+{
+        GtkEntryIconPosition position;
+        gboolean sensitive;
+        const gchar *text;
+
+        text = gtk_entry_get_text (entry);
+        position = GTK_ENTRY_ICON_SECONDARY;
+        sensitive = (text != NULL) && (*text != '\0');
+
+        gtk_entry_set_icon_sensitive (entry, position, sensitive);
+}
+
+/**
+ * gva_main_search_entry_icon_release_cb:
+ * @entry: the search entry
+ * @position: the position of the clicked icon
+ * @event: the button release event
+ *
+ * Handler for #GtkEntry::icon-release signals to the search entry.
+ *
+ * If @position is %GTK_ENTRY_ICON_SECONDARY, clears the search entry.
+ **/
+void
+gva_main_search_entry_icon_release_cb (GtkEntry *entry,
+                                       GtkEntryIconPosition position,
+                                       GdkEvent *event)
+{
+        if (position == GTK_ENTRY_ICON_SECONDARY)
+                gva_main_clear_search ();
 }
 
 /**
