@@ -1,4 +1,4 @@
-/* Copyright 2007, 2008 Matthew Barnes
+/* Copyright 2007-2010 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -21,7 +21,7 @@
 #include <errno.h>
 #include <signal.h>
 #include <string.h>
-#include <wait.h>
+#include <sys/wait.h>
 
 #include "gva-error.h"
 #include "gva-util.h"
@@ -152,7 +152,7 @@ process_exited (GPid pid,
         }
 }
 
-static void
+static GIOStatus
 process_read_line (GvaProcess *process,
                    GIOChannel *channel,
                    GQueue *queue,
@@ -171,10 +171,9 @@ process_read_line (GvaProcess *process,
                 g_signal_emit (process, signal_id, 0);
         }
         else
-        {
-                g_assert (error != NULL);
                 process_propagate_error (process, error);
-        }
+
+        return status;
 }
 
 static gboolean
@@ -182,6 +181,12 @@ process_stdout_ready (GIOChannel *channel,
                       GIOCondition condition,
                       GvaProcess *process)
 {
+        /* The logic here is tricky to get right across all operating
+         * systems due to the various ways poll() can report end-of-file.
+         * See http://www.greenend.org.uk/rjk/2001/06/poll.html. */
+
+        GIOStatus status;
+
         if (condition & G_IO_IN)
         {
                 /* For better performance, keep reading lines as long as
@@ -192,17 +197,23 @@ process_stdout_ready (GIOChannel *channel,
 
                 do
                 {
-                        process_read_line (
+                        status = process_read_line (
                                 process, channel,
                                 process->priv->stdout_lines,
                                 signals[STDOUT_READY]);
 
+                        if (status != G_IO_STATUS_NORMAL)
+                                break;
+
+                        /* Continue reading as long as data is available
+                         * in the internal buffer. */
                         condition =
                                 g_io_channel_get_buffer_condition (channel);
                 }
-                while (condition & G_IO_IN);
+                while (condition == G_IO_IN);
 
-                return TRUE;
+                if (status == G_IO_STATUS_NORMAL)
+                        return TRUE;
         }
 
         process->priv->stdout_source_id = 0;
@@ -215,16 +226,19 @@ process_stderr_ready (GIOChannel *channel,
                       GIOCondition condition,
                       GvaProcess *process)
 {
+        GIOStatus status;
+
         if (condition & G_IO_IN)
         {
                 /* Do NOT loop here, as we do for stdout. */
 
-                process_read_line (
+                status = process_read_line (
                         process, channel,
                         process->priv->stderr_lines,
                         signals[STDERR_READY]);
 
-                return TRUE;
+                if (status == G_IO_STATUS_NORMAL)
+                        return TRUE;
         }
 
         process->priv->stderr_source_id = 0;
@@ -232,6 +246,25 @@ process_stderr_ready (GIOChannel *channel,
         return FALSE;
 }
 
+static void
+process_debug_message (GvaProcess *process,
+                       const gchar *line,
+                       const gchar *sep)
+{
+        GPid pid;
+        gchar *copy;
+
+        /* Copy the string just to remove the newline. */
+        pid = gva_process_get_pid (process);
+        copy = g_strchomp (g_strdup (line));
+
+        g_log (
+                G_LOG_DOMAIN, GVA_DEBUG_IO,
+                "Process %d %s %s", (gint) pid, sep, copy);
+
+        g_free (copy);
+}
+
 static GObject *
 process_constructor (GType type,
                      guint n_construct_properties,
@@ -418,7 +451,7 @@ process_class_init (GvaProcessClass *class)
         class->stderr_read_line = process_stderr_read_line;
 
         /**
-         * GvaProcess:pid:
+         * GvaProcess:pid
          *
          * The ID of the child process.
          **/
@@ -434,7 +467,7 @@ process_class_init (GvaProcessClass *class)
                         G_PARAM_CONSTRUCT_ONLY));
 
         /**
-         * GvaProcess:stdin:
+         * GvaProcess:stdin
          *
          * The file descriptor for the child process' stdin pipe.
          **/
@@ -450,7 +483,7 @@ process_class_init (GvaProcessClass *class)
                         G_PARAM_CONSTRUCT_ONLY));
 
         /**
-         * GvaProcess:stdout:
+         * GvaProcess:stdout
          *
          * The file descriptor for the child process' stdout pipe.
          **/
@@ -466,7 +499,7 @@ process_class_init (GvaProcessClass *class)
                         G_PARAM_CONSTRUCT_ONLY));
 
         /**
-         * GvaProcess:stderr:
+         * GvaProcess:stderr
          *
          * The file descriptor for the child process' stderr pipe.
          **/
@@ -482,7 +515,7 @@ process_class_init (GvaProcessClass *class)
                         G_PARAM_CONSTRUCT_ONLY));
 
         /**
-         * GvaProcess:priority:
+         * GvaProcess:priority
          *
          * Priority of the event sources that watch for incoming data.
          **/
@@ -499,7 +532,7 @@ process_class_init (GvaProcessClass *class)
                         G_PARAM_CONSTRUCT_ONLY));
 
         /**
-         * GvaProcess:progress:
+         * GvaProcess:progress
          *
          * Progress value, the meaning of which is defined by the
          * application.
@@ -716,6 +749,8 @@ gva_process_write_stdin (GvaProcess *process,
         g_return_val_if_fail (GVA_IS_PROCESS (process), FALSE);
         g_return_val_if_fail (data != NULL, FALSE);
 
+        process_debug_message (process, data, "<<<");
+
         while (status == G_IO_STATUS_AGAIN)
                 status = g_io_channel_write_chars (
                         process->priv->stdin_channel, data,
@@ -786,12 +821,17 @@ gchar *
 gva_process_stdout_read_line (GvaProcess *process)
 {
         GvaProcessClass *class;
+        gchar *line;
 
         g_return_val_if_fail (GVA_IS_PROCESS (process), NULL);
 
         class = GVA_PROCESS_GET_CLASS (process);
         g_return_val_if_fail (class->stdout_read_line != NULL, NULL);
-        return class->stdout_read_line (process);
+        line = class->stdout_read_line (process);
+
+        process_debug_message (process, line, ">>>");
+
+        return line;
 }
 
 /**
@@ -810,12 +850,17 @@ gchar *
 gva_process_stderr_read_line (GvaProcess *process)
 {
         GvaProcessClass *class;
+        gchar *line;
 
         g_return_val_if_fail (GVA_IS_PROCESS (process), NULL);
 
         class = GVA_PROCESS_GET_CLASS (process);
         g_return_val_if_fail (class->stderr_read_line != NULL, NULL);
-        return class->stderr_read_line (process);
+        line = class->stderr_read_line (process);
+
+        process_debug_message (process, line, "!!!");
+
+        return line;
 }
 
 /**
