@@ -1,4 +1,4 @@
-/* Copyright 2007, 2008 Matthew Barnes
+/* Copyright 2007-2010 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -25,6 +25,8 @@
 #include "gva-error.h"
 #include "gva-game-store.h"
 #include "gva-history.h"
+#include "gva-music-button.h"
+#include "gva-preferences.h"
 #include "gva-tree-view.h"
 #include "gva-ui.h"
 #include "gva-util.h"
@@ -35,6 +37,16 @@
 #define SQL_SELECT_NAME \
         "SELECT * FROM available WHERE name = \"%s\""
 
+#define SQL_SELECT_PARENT \
+        "SELECT game.name, game.description, available.name NOTNULL " \
+        "FROM game LEFT JOIN available ON game.name = available.name " \
+        "WHERE game.name = \"%s\" ORDER BY game.description"
+
+#define SQL_SELECT_CLONES \
+        "SELECT game.name, game.description, available.name NOTNULL " \
+        "FROM game LEFT JOIN available ON game.name = available.name " \
+        "WHERE game.cloneof = \"%s\" ORDER BY game.description"
+
 #define SQL_SELECT_CPU \
         "SELECT COUNT(*), name, clock FROM chip " \
         "WHERE game = \"%s\" AND type = \"cpu\" " \
@@ -81,8 +93,73 @@ static const gchar *video_labels[] =
         "properties-video3-label"
 };
 
+static const gchar *current_game;
 static guint update_timeout_source_id;
 
+static void
+properties_scroll_to_top (void)
+{
+        GtkScrolledWindow *scrolled_window;
+        GtkAdjustment *adjustment;
+
+        scrolled_window = GTK_SCROLLED_WINDOW (
+                GVA_WIDGET_PROPERTIES_HISTORY_SCROLLED_WINDOW);
+        adjustment = gtk_scrolled_window_get_vadjustment (scrolled_window);
+        gtk_adjustment_set_value (adjustment, 0.0);
+
+        scrolled_window = GTK_SCROLLED_WINDOW (
+                GVA_WIDGET_PROPERTIES_TECHNICAL_SCROLLED_WINDOW);
+        adjustment = gtk_scrolled_window_get_vadjustment (scrolled_window);
+        gtk_adjustment_set_value (adjustment, 0.0);
+}
+
+static void
+properties_link_clicked_cb (GtkLabel *label,
+                            const gchar *game)
+{
+        GtkWidget *widget;
+
+        widget = GVA_WIDGET_PROPERTIES_TECHNICAL_SCROLLED_WINDOW;
+        gtk_widget_grab_focus (widget);
+
+        gva_properties_show_game (game);
+        gva_tree_view_set_selected_game (game);
+}
+
+static void
+properties_add_game_label (GtkBox *box,
+                           const gchar *game,
+                           const gchar *description,
+                           gboolean available)
+{
+        GtkWidget *widget;
+
+        widget = gtk_label_new (NULL);
+        gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
+
+        if (available)
+        {
+                gchar *markup;
+
+                markup = g_markup_printf_escaped (
+                        "<a href=\"%s\" title=\"%s\">%s</a>",
+                        game, _("Show this game"), description);
+
+                gtk_label_set_markup (GTK_LABEL (widget), markup);
+
+                g_signal_connect (
+                        widget, "activate-link",
+                        G_CALLBACK (properties_link_clicked_cb), NULL);
+
+                g_free (markup);
+        }
+        else
+                gtk_label_set_text (GTK_LABEL (widget), description);
+
+        gtk_box_pack_start (box, widget, FALSE, FALSE, 0);
+        gtk_widget_show (widget);
+}
+
 static gchar *
 properties_cpu_description (const gchar *name,
                             const gchar *clock,
@@ -144,6 +221,115 @@ properties_update_bios (GtkTreeModel *model,
         g_free (bios);
 }
 
+static void
+properties_update_clones (GtkTreeModel *model,
+                          GtkTreeIter *iter)
+{
+        GtkBox *box;
+        GList *children;
+        gchar **result = NULL;
+        gchar *game;
+        gint rows, columns, ii;
+        GError *error = NULL;
+
+        box = GTK_BOX (GVA_WIDGET_PROPERTIES_ORIGINAL_LINKS);
+        children = gtk_container_get_children (GTK_CONTAINER (box));
+        g_list_foreach (children, (GFunc) gtk_widget_destroy, NULL);
+        g_list_free (children);
+
+        gtk_tree_model_get (
+                model, iter, GVA_GAME_STORE_COLUMN_CLONEOF, &game, -1);
+
+        if (game != NULL)
+        {
+                gchar *sql;
+
+                sql = g_strdup_printf (SQL_SELECT_PARENT, game);
+                if (!gva_db_get_table (sql, &result, &rows, &columns, &error))
+                {
+                        gva_error_handle (&error);
+                        rows = columns = 0;
+                }
+                g_free (sql);
+                g_free (game);
+        }
+        else
+                rows = columns = 0;
+
+        if (rows > 0)
+                gtk_widget_show (GVA_WIDGET_PROPERTIES_ORIGINAL_VBOX);
+        else
+                gtk_widget_hide (GVA_WIDGET_PROPERTIES_ORIGINAL_VBOX);
+
+        for (ii = 0; ii < rows; ii++)
+        {
+                const gchar *parent_name;
+                const gchar *description;
+                gboolean available;
+                gchar **values;
+
+                values = result + ((ii + 1) * columns);
+
+                parent_name = values[0];
+                description = values[1];
+                available = (*values[2] == '1');
+
+                properties_add_game_label (
+                        box, parent_name, description, available);
+        }
+
+        g_strfreev (result);
+        result = NULL;
+
+        box = GTK_BOX (GVA_WIDGET_PROPERTIES_ALTERNATE_LINKS);
+        children = gtk_container_get_children (GTK_CONTAINER (box));
+        g_list_foreach (children, (GFunc) gtk_widget_destroy, NULL);
+        g_list_free (children);
+
+        gtk_tree_model_get (
+                model, iter, GVA_GAME_STORE_COLUMN_NAME, &game, -1);
+
+        if (game != NULL)
+        {
+                gchar *sql;
+
+                sql = g_strdup_printf (SQL_SELECT_CLONES, game);
+                if (!gva_db_get_table (sql, &result, &rows, &columns, &error))
+                {
+                        gva_error_handle (&error);
+                        rows = columns = 0;
+                }
+                g_free (sql);
+                g_free (game);
+        }
+        else
+                rows = columns = 0;
+
+        if (rows > 0)
+                gtk_widget_show (GVA_WIDGET_PROPERTIES_ALTERNATE_VBOX);
+        else
+                gtk_widget_hide (GVA_WIDGET_PROPERTIES_ALTERNATE_VBOX);
+
+        for (ii = 0; ii < rows; ii++)
+        {
+                const gchar *clone_name;
+                const gchar *description;
+                gboolean available;
+                gchar **values;
+
+                values = result + ((ii + 1) * columns);
+
+                clone_name = values[0];
+                description = values[1];
+                available = (*values[2] == '1');
+
+                properties_add_game_label (
+                        box, clone_name, description, available);
+        }
+
+        g_strfreev (result);
+}
+
 static void
 properties_update_cpu (const gchar *name)
 {
@@ -158,7 +344,8 @@ properties_update_cpu (const gchar *name)
         sql = g_strdup_printf (
                 SQL_SELECT_CPU, name, G_N_ELEMENTS (cpu_labels));
 
-        if (!gva_db_get_table (sql, &result, &rows, &columns, &error)) {
+        if (!gva_db_get_table (sql, &result, &rows, &columns, &error))
+        {
                 gva_error_handle (&error);
                 rows = columns = 0;
         }
@@ -283,6 +470,19 @@ properties_update_history (GtkTreeModel *model,
 #endif
 }
 
+static void
+properties_update_music (const gchar *name)
+{
+        GvaMusicButton *music_button;
+
+        music_button = GVA_MUSIC_BUTTON (GVA_WIDGET_PROPERTIES_MUSIC_BUTTON);
+        gva_music_button_set_game (music_button, name);
+
+        if (GTK_WIDGET_VISIBLE (GVA_WIDGET_PROPERTIES_WINDOW)
+                && gva_preferences_get_auto_play ())
+                gva_music_button_play (music_button);
+}
+
 static void
 properties_update_sound (const gchar *name)
 {
@@ -373,14 +573,14 @@ properties_update_status (GtkTreeModel *model,
         else
                 stock_id = GTK_STOCK_DIALOG_WARNING;
 
-        widget = GVA_WIDGET_PROPERTIES_STATUS_HBOX;
+        widget = GVA_WIDGET_PROPERTIES_STATUS_FRAME;
         visible = (strcmp (driver_status, "imperfect") == 0) ||
                 (strcmp (driver_status, "preliminary") == 0);
         g_object_set (widget, "visible", visible, NULL);
 
         gtk_image_set_from_stock (
                 GTK_IMAGE (GVA_WIDGET_PROPERTIES_STATUS_IMAGE),
-                stock_id, GTK_ICON_SIZE_DIALOG);
+                stock_id, GTK_ICON_SIZE_DND);
         g_object_set (widget, "visible", visible, NULL);
 
         widget = GVA_WIDGET_PROPERTIES_IMPERFECT_COLOR_LABEL;
@@ -508,40 +708,12 @@ properties_update_video (const gchar *name)
 static gboolean
 properties_update_timeout_cb (void)
 {
-        GtkTreeModel *model;
         const gchar *name;
-        gchar *sql;
-        GError *error = NULL;
 
         name = gva_tree_view_get_selected_game ();
 
-        /* Leave the window alone if a game is not selected. */
-        if (name == NULL)
-                return FALSE;
-
-        sql = g_strdup_printf (SQL_SELECT_NAME, name);
-        model = gva_game_store_new_from_query (sql, &error);
-        gva_error_handle (&error);
-        g_free (sql);
-
-        if (model != NULL)
-        {
-                GtkTreeIter iter;
-                gboolean valid;
-
-                valid = gtk_tree_model_get_iter_first (model, &iter);
-                g_assert (valid);
-
-                properties_update_bios (model, &iter);
-                properties_update_cpu (name);
-                properties_update_header (model, &iter);
-                properties_update_history (model, &iter);
-                properties_update_sound (name);
-                properties_update_status (model, &iter);
-                properties_update_video (name);
-
-                g_object_unref (model);
-        }
+        if (name != NULL)
+                gva_properties_show_game (name);
 
         update_timeout_source_id = 0;
 
@@ -561,6 +733,25 @@ properties_selection_changed_cb (GtkTreeSelection *selection)
                 properties_update_timeout_cb, NULL);
 }
 
+static void
+properties_notify_music_status_cb (GvaMusicButton *music_button)
+{
+        GtkWidget *label;
+        const gchar *status;
+        gchar *markup;
+
+        label = GVA_WIDGET_PROPERTIES_MUSIC_STATUS;
+        status = gva_music_button_get_status (music_button);
+
+        /* Use whitespace to keep the widget height stable. */
+        if (status == NULL)
+                status = " ";
+
+        markup = g_markup_printf_escaped ("<small>%s</small>", status);
+        gtk_label_set_markup (GTK_LABEL (label), markup);
+        g_free (markup);
+}
+
 /**
  * gva_properties_init:
  *
@@ -571,18 +762,19 @@ properties_selection_changed_cb (GtkTreeSelection *selection)
 void
 gva_properties_init (void)
 {
-        GtkWindow *window;
         GtkTreeView *view;
+        GtkSettings *settings;
         GtkWidget *text_view;
+        GHashTable *color_hash;
         PangoFontDescription *desc;
+        const GdkColor *color;
+        GtkWidget *widget;
         gchar *font_name;
 
 #ifndef HISTORY_FILE
         GtkNotebook *notebook;
-        GtkWidget *page;
 #endif
 
-        window = GTK_WINDOW (GVA_WIDGET_PROPERTIES_WINDOW);
         view = GTK_TREE_VIEW (GVA_WIDGET_MAIN_TREE_VIEW);
         text_view = GVA_WIDGET_PROPERTIES_HISTORY_TEXT_VIEW;
 
@@ -602,32 +794,128 @@ gva_properties_init (void)
                 gtk_tree_view_get_selection (view), "changed",
                 G_CALLBACK (properties_selection_changed_cb), NULL);
 
+        g_signal_connect (
+                GVA_WIDGET_PROPERTIES_MUSIC_BUTTON, "notify::status",
+                G_CALLBACK (properties_notify_music_status_cb), NULL);
+
+        gconf_bridge_bind_window (
+                gconf_bridge_get (), GVA_GCONF_PROPERTIES_PREFIX,
+                GTK_WINDOW (GVA_WIDGET_PROPERTIES_WINDOW), TRUE, FALSE);
+
         font_name = gva_get_monospace_font_name ();
         desc = pango_font_description_from_string (font_name);
         gtk_widget_modify_font (text_view, desc);
         pango_font_description_free (desc);
         g_free (font_name);
 
+        settings = gtk_settings_get_default ();
+        g_object_get (settings, "color-hash", &color_hash, NULL);
+        color = g_hash_table_lookup (color_hash, "tooltip_bg_color");
+        widget = GVA_WIDGET_PROPERTIES_STATUS_EVENT_BOX;
+        gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, color);
+        widget = GVA_WIDGET_PROPERTIES_STATUS_FRAME;
+        gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, color);
+
+#ifndef HAVE_GSTREAMER
+        gtk_widget_hide (GVA_WIDGET_PROPERTIES_MUSIC_TABLE);
+#endif
+
 #ifndef HISTORY_FILE
         /* Hide the history page if we have no history file. */
         notebook = GTK_NOTEBOOK (GVA_WIDGET_PROPERTIES_NOTEBOOK);
-        page = gtk_notebook_get_nth_page (notebook, NOTEBOOK_PAGE_HISTORY);
-        gtk_widget_hide (page);
+        widget = gtk_notebook_get_nth_page (notebook, NOTEBOOK_PAGE_HISTORY);
+        gtk_widget_hide (widget);
 #endif
 }
 
 /**
- * gva_properties_close_clicked_cb:
+ * gva_properties_show_game:
+ * @game: the name of a game
+ *
+ * Shows information about @game in the Properties window.
+ **/
+void
+gva_properties_show_game (const gchar *game)
+{
+        GtkTreeModel *model;
+        gchar *sql;
+        GError *error = NULL;
+
+        g_return_if_fail (game != NULL);
+
+        game = g_intern_string (game);
+
+        /* Refreshing the Properties window is expensive.  If the
+         * requested game is already being shown, do nothing. */
+        if (game == current_game)
+                return;
+
+        sql = g_strdup_printf (SQL_SELECT_NAME, game);
+        model = gva_game_store_new_from_query (sql, &error);
+        gva_error_handle (&error);
+        g_free (sql);
+
+        if (model != NULL)
+        {
+                GtkTreeIter iter;
+                gboolean valid;
+
+                valid = gtk_tree_model_get_iter_first (model, &iter);
+                g_assert (valid);
+
+                current_game = game;
+                properties_scroll_to_top ();
+
+                properties_update_bios (model, &iter);
+                properties_update_clones (model, &iter);
+                properties_update_cpu (game);
+                properties_update_header (model, &iter);
+                properties_update_history (model, &iter);
+                properties_update_music (game);
+                properties_update_sound (game);
+                properties_update_status (model, &iter);
+                properties_update_video (game);
+
+                g_object_unref (model);
+        }
+}
+
+/**
+ * gva_properties_hide_cb:
  * @window: the "Properties" window
  * @button: the "Close" button
  *
- * Handler for #GtkButton::clicked signals to the "Close" button.
+ * Handler for #GtkWidget::show signals to the "Properties" window.
  *
- * Hides @window.
+ * Stops in-game music clip.
  **/
 void
-gva_properties_close_clicked_cb (GtkWindow *window,
-                                 GtkButton *button)
+gva_properties_hide_cb (GtkWindow *window)
 {
-        gtk_widget_hide (GTK_WIDGET (window));
+        GvaMusicButton *music_button;
+
+        music_button = GVA_MUSIC_BUTTON (GVA_WIDGET_PROPERTIES_MUSIC_BUTTON);
+        gva_music_button_pause (music_button);
+}
+
+/**
+ * gva_properties_show_cb:
+ * @window: the "Properties" window
+ *
+ * Handler for #GtkWidget::show signals to the "Properties" window.
+ *
+ * Resets all scrolled windows to the top, and starts in-game music clip
+ * if the "auto-play" preference is enabled.
+ **/
+void
+gva_properties_show_cb (GtkWindow *window)
+{
+        GvaMusicButton *music_button;
+
+        music_button = GVA_MUSIC_BUTTON (GVA_WIDGET_PROPERTIES_MUSIC_BUTTON);
+
+        if (gva_preferences_get_auto_play ())
+                gva_music_button_play (music_button);
+
+        properties_scroll_to_top ();
 }
